package com.yunzhi.questions.common;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.api.impl.WxMaServiceImpl;
import cn.binarywang.wx.miniapp.bean.WxMaCodeLineColor;
import cn.binarywang.wx.miniapp.bean.WxMaSubscribeMessage;
import cn.binarywang.wx.miniapp.config.WxMaConfig;
import cn.binarywang.wx.miniapp.config.impl.WxMaDefaultConfigImpl;
import com.github.binarywang.wxpay.config.WxPayConfig;
import com.github.binarywang.wxpay.constant.WxPayConstants;
import com.github.binarywang.wxpay.service.WxPayService;
import com.github.binarywang.wxpay.service.impl.WxPayServiceImpl;
import com.yunzhi.questions.config.WeixinConfig;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.api.impl.WxMpServiceImpl;
import me.chanjar.weixin.mp.bean.template.WxMpTemplateData;
import me.chanjar.weixin.mp.bean.template.WxMpTemplateMessage;
import me.chanjar.weixin.mp.config.WxMpConfigStorage;
import me.chanjar.weixin.mp.config.impl.WxMpDefaultConfigImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import java.io.File;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Slf4j
@Component
public class WxUtils {
    // 小程序
    private Map<String, WxMaService> maServiceMap;
    // 公众号
    private Map<String, WxMpService> mpServiceMap;

    @Autowired
    public WxUtils() {
        maServiceMap = new ConcurrentHashMap<>();
        mpServiceMap = new ConcurrentHashMap<>();
    }

    /**
     * 生成 公众号 服务
     * @param config
     * @return
     */
    public WxMpService newMpService(WxMpConfigStorage config) {
        WxMpService service = new WxMpServiceImpl();
        service.setWxMpConfigStorage(config);
        return service;
    }

    /**
     * 添加一个公众号服务到 Map
     * @param appid
     * @param service
     */
    public void addMpService(String appid, WxMpService service) {
        mpServiceMap.put(appid, service);
    }

    /**
     * 删除一个公众号服务
     * @param appid
     */
    public void removeMpService(String appid) {
        mpServiceMap.remove(appid);
    }

    /**
     * 生成一个小程序服务
     * @param config
     * @return
     */
    public WxMaService newMaService(WxMaConfig config) {
        WxMaService service = new WxMaServiceImpl();
        service.setWxMaConfig(config);
        return service;
    }

    /**
     * 添加一个小程序服务到 MAP
     * @param appid
     * @param service
     */
    public void addMaService(String appid, WxMaService service) {
        maServiceMap.put(appid, service);
    }

    /**
     * 删除一个小程序服务
     * @param appid
     */
    public void removeMaService(String appid) {
        maServiceMap.remove(appid);
    }

    /**
     * 生成一个小程序二维码
     * @param service
     * @param path
     * @return
     * @throws WxErrorException
     */
    public File createMaQRCode(WxMaService service, String path) throws WxErrorException {
        return service.getQrcodeService().createQrcode(path);
    }

    /**
     * 获取 appid 对应的小程序服务
     * @param appid
     * @return
     */
    public WxMaService getMaService(String appid) {
        return maServiceMap.get(appid);
    }

    /**
     * 获取 appid 对应的公众号服务
     * @param appid
     * @return
     */
    public WxMpService getMpService(String appid) {
        return mpServiceMap.get(appid);
    }


    @Data
    public static class MAQrCodeParam {
        private String scene;
        private String page;
        private int width;
        private boolean autoColor;
        private WxMaCodeLineColor lineColor;
        private boolean isHyaline;
    }

    /**
     * 发送小程序订阅消息
     * @param templateId
     * @param openid
     * @param page
     * @param dataList
     * @throws Exception
     */
    @Async
    public void sendMaMessage(WxMaService service, String templateId, String openid, String page, List<WxMaSubscribeMessage.Data> dataList) throws Exception {
        WxMaSubscribeMessage wxMaSubscribeMessage = WxMaSubscribeMessage
                .builder()
                .templateId(templateId)
                .toUser(openid)
                .page(page)
                .data(dataList)
                .build();

        try {
            service.getMsgService().sendSubscribeMsg(wxMaSubscribeMessage);
        } catch (Exception e) {
            e.printStackTrace();

            // 再发送一次
            Thread.sleep(1000);
            service.getMsgService().sendSubscribeMsg(wxMaSubscribeMessage);
        }
    }

    /**
     * 发送公众号模板消息
     * @param toUser
     * @param templateId
     * @param miniAppId
     * @param page
     * @param datas
     * @return
     * @throws WxErrorException
     */
    public void sendMpMessage(WxMpService service, String toUser, String templateId, String miniAppId, String page, String url, List<WxMpTemplateData> datas) throws Exception {
        String pagepath = StringUtils.trim(page, "/");
        WxMpTemplateMessage.MiniProgram miniProgram = null == miniAppId ? null : new WxMpTemplateMessage.MiniProgram(miniAppId, pagepath, false);

        log.info("模板信息-内容: {}", miniProgram);

        WxMpTemplateMessage templateMessage = WxMpTemplateMessage.builder()
                .toUser(toUser)
                .miniProgram(miniProgram)
                .templateId(templateId)
                .url(url)
                .build();

        for (WxMpTemplateData data: datas) {
            templateMessage.addData(data);
        }

        try {
            service.getTemplateMsgService().sendTemplateMsg(templateMessage);
        } catch (Exception e) {
            e.printStackTrace();

            // 再发送一次
            Thread.sleep(1000);
            service.getTemplateMsgService().sendTemplateMsg(templateMessage);
        }
    }
}
