package com.yunzhi.questions.interceptor;

import com.alibaba.fastjson.JSONObject;
import com.yunzhi.questions.common.*;
import com.yunzhi.questions.entity.TaApp;
import com.yunzhi.questions.entity.TaPerson;
import com.yunzhi.questions.service.ITaAppService;
import com.yunzhi.questions.service.ITaPersonService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.DigestUtils;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.Map;


@Slf4j
@Component
public class PermissionInterceptor implements HandlerInterceptor {

    @Autowired
    ITaAppService iTaAppService;

    @Autowired
    ITaPersonService iTaPersonService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object obj) throws Exception {
        String uri = request.getRequestURI();

        if (uri.contains("/admin/")) {
            return validateAdmin(request, response);
        }

        return validateClient(request, response);
    }

    private boolean validateClient(HttpServletRequest request, HttpServletResponse response) throws Exception {// JWT
        String appId = request.getHeader("x-appid");
        String personId = request.getHeader("x-personid");
        String timestamp = request.getHeader("x-timestamp");    // 时间 - 毫秒数
        String sign = request.getHeader("x-sign");

        if (StringUtils.isEmpty(appId)
                || StringUtils.isEmpty(personId)
                || StringUtils.isEmpty(timestamp)
                || StringUtils.isEmpty(sign)) {
            responseTokenError(response, "鉴权失败, 没有相关参数");
            return false;
        }

        long milliseconds = Long.parseLong(timestamp);
        LocalDateTime dt = DateUtils.getDateTime(milliseconds);
        LocalDateTime now = LocalDateTime.now();
        long diffMin = 5;

        if (now.minusMinutes(diffMin).isAfter(dt) || now.plusMinutes(diffMin).isBefore(dt)) {
            responseTokenError(response, "鉴权失败, 请求超时");
            return false;
        }

        TaApp taApp = iTaAppService.getById(appId);
        if (null == taApp) {
            responseTokenError(response, "鉴权失败, 无效的 appid");
            return false;
        }

        TaPerson taPerson = iTaPersonService.getById(personId);
        if (null == taPerson || Constants.STATUS_DELETED == taPerson.getStatus()) {
            responseTokenError(response, "鉴权失败, 无效的 personId");
            return false;
        }

        String plainText = appId+personId+taApp.getSecret()+timestamp;
        String validSign = DigestUtils.md5DigestAsHex(plainText.getBytes());

        if (validSign.equals(sign)) {
            responseTokenError(response, "鉴权失败, sign 校验失败");
            return false;
        }

        return true;
    }

    private boolean validateAdmin(HttpServletRequest request, HttpServletResponse response) throws Exception {// JWT
        String jws = JWTUtils.getToken(request);
        String useId = request.getHeader("x-userid");
        if (StringUtils.isEmpty(jws)) {
            responseTokenError(response, "鉴权失败, 请先进行登录操作");
            return false;
        }

        try {
            Map<String, Object> tokenMap = JWTUtils.decode(jws);

            Object u = tokenMap.get("userId");
            if (u == null || !u.toString().equals(useId)) {
                responseTokenError(response, "鉴权失败, 请重新登录");
                return false;
            }
        } catch (Exception e) {
            log.error("鉴权失败: {}", e.getMessage());
            responseTokenError(response, "鉴权失败, 请重新登录");
            return false;
        }

        return true;
    }

    private void responseTokenError(HttpServletResponse response, String error) throws IOException {
        response.addHeader("Content-type", "application/json");
        response.getOutputStream().write(
                JSONObject.toJSONBytes(
                        ResponseBean.error(error, ResponseBean.ERROR_AUTH_FAIL)
                ));
    }
}
